/*****************************************************************************
 *          DigASM: a minimal assembler for educational purpose              *
 ***************************************************************************** 
 *                                                                           *
 * Copyright (c) 2010 Christophe Alias                                       *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License as published by the     *
 * Free Software Foundation; either version 2 of the License, or (at your    * 
 * option) any later version.                                                *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General  *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License along   *
 * with this program; if not, write to the Free Software Foundation, Inc.,   *
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                 *
 *                                                                           *
 * Written by Christophe Alias, Christophe.Alias@ens-lyon.fr                 *
 *                                                                           *
 *****************************************************************************/
/* ============================================================== */
/*                             C PRELUDE                          */
/* ============================================================== */

%{
#include <stdlib.h>
#include <stdio.h>
#include "code_generation.h"   

int		line_number	= 1;	/* number of current source line */
extern int	yylex();	/* lexical analyzer generated from lex.l */
extern char	*yytext;	/* last token, defined in lex.l  */

void yyerror(char *s)
  { fprintf(stderr, 
	    "digasm:%d: syntax error. Last token was \"%s\"\n",
	    line_number, yytext);
    exit(1);
  }
void error(char *s)
  { fprintf(stderr, 
	    "digasm:%d: %s\n", line_number, s);
    exit(1);
  }

%}

/* ============================================================== */
/*                     TYPES OF NON-TERMINALS                     */         
/* ============================================================== */

%union
{
  char* id_string;
  unsigned short int integer;
}


/* ============================================================== */
/*                       TOKENS DECLARATION
/* ============================================================== */

%token TK_INT TK_ID TK_CAR TK_STRING

/* dlimiteurs */
%token TK_LBRAC TK_RBRAC TK_LPAR TK_RPAR
%token TK_COMMA TK_COLON

/* Oprateurs arithmtiques */
%token TK_PLUS TK_MINUS

/* Mnmoniques */
%token TK_ADD TK_SUB TK_LDI TK_LD TK_ST TK_BEQ TK_J TK_JA
%token TK_REG

/* Labels operations */
%token TK_LO TK_HI

/* Static data declaration */
%token TK_DATA

%type<integer> TK_INT
%type<id_string> TK_ID
%type<integer> TK_CAR
%type<id_string> TK_STRING
%type<integer> TK_REG

/* axiom */
%start program

%%

/* ============================================================== */
/*               GRAMMAR ITSELF WITH SEMANTIC ACTIONS             */
/* ============================================================== */

instruction:

/* -------------------------------------------------------------- */
/*             Oprations arithmetiques (ADD, SUB)                */
/* -------------------------------------------------------------- */

  TK_ADD TK_REG TK_COMMA TK_REG TK_COMMA TK_REG
    { emit_add($2,$4,$6); }
| TK_SUB TK_REG TK_COMMA TK_REG TK_COMMA TK_REG
    { emit_sub($2,$4,$6); }

/* -------------------------------------------------------------- */
/*                   Chargement immdiat (LDI)                    */
/* -----------------------88--------------------------------------- */

| TK_LDI TK_REG TK_COMMA TK_INT
    { emit_ldi($2,$4); }
| TK_LDI TK_REG TK_COMMA TK_CAR
    { emit_ldi($2,$4); }
| TK_LDI TK_REG TK_COMMA TK_LO TK_LPAR TK_ID TK_RPAR
    { emit_ldi_label_lo($2,$6); }
| TK_LDI TK_REG TK_COMMA TK_HI TK_LPAR TK_ID TK_RPAR
    { emit_ldi_label_hi($2,$6); }

/* -------------------------------------------------------------- */
/*                         LOAD/STORE                             */
/* -------------------------------------------------------------- */

| TK_LD TK_REG TK_COMMA TK_LBRAC TK_REG TK_PLUS TK_INT TK_RBRAC
    { emit_ld($2,$5,$7); }
| TK_LD TK_REG TK_COMMA TK_LBRAC TK_REG TK_MINUS TK_INT TK_RBRAC
    { emit_ld_neg($2,$5,$7); }
| TK_LD TK_REG TK_COMMA TK_LBRAC TK_REG TK_RBRAC
    { emit_ld($2,$5,0); }

| TK_ST TK_REG TK_COMMA TK_LBRAC TK_REG TK_PLUS TK_INT TK_RBRAC
    { emit_st($2,$5,$7); }
| TK_ST TK_REG TK_COMMA TK_LBRAC TK_REG TK_MINUS TK_INT TK_RBRAC
    { emit_st_neg($2,$5,$7); }
| TK_ST TK_REG TK_COMMA TK_LBRAC TK_REG TK_RBRAC
    { emit_st($2,$5,0); }

/* -------------------------------------------------------------- */
/*                           SAUTS                                */
/* -------------------------------------------------------------- */

| TK_BEQ TK_REG TK_COMMA TK_REG TK_COMMA TK_ID
    { emit_beq($2,$4,$6); }
| TK_JA TK_REG TK_COMMA TK_REG
    { emit_ja($2,$4); }
| TK_J TK_ID
    { emit_j($2); }

| TK_ID TK_COLON
    { define_label($1); }


/* -------------------------------------------------------------- */
/*                      DONNEES STATIQUES                         */
/* -------------------------------------------------------------- */

| TK_DATA TK_INT
    { emit_data_num1($2); }
| TK_DATA TK_INT TK_COMMA TK_INT
    { emit_data_num2($2,$4); }
| TK_DATA TK_STRING 
    { emit_data_string($2); }
;

/* -------------------------------------------------------------- */
/*      5/ *Main production* (program=declarations+statements)    */
/* -------------------------------------------------------------- */

program: 
|program instruction
;
